@@ -58,6 +58,7 @@ class RasterDataset(XRasterBase):
     def reproject(
         self,
         dst_crs: Any,
+        *,
         resolution: Optional[Union[float, tuple[float, float]]] = None,
         shape: Optional[tuple[int, int]] = None,
         transform: Optional[Affine] = None,
@@ -116,7 +117,7 @@ class RasterDataset(XRasterBase):
         resampled_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
             try:
-                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                x_dim, y_dim = _get_spatial_dims(self._obj, var=var)
                 resampled_dataset[var] = (
                     self._obj[var]
                     .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
@@ -141,6 +142,7 @@ class RasterDataset(XRasterBase):
     def reproject_match(
         self,
         match_data_array: Union[xarray.DataArray, xarray.Dataset],
+        *,
         resampling: Resampling = Resampling.nearest,
         **reproject_kwargs,
     ) -> xarray.Dataset:
@@ -174,7 +176,7 @@ class RasterDataset(XRasterBase):
         resampled_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
             try:
-                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                x_dim, y_dim = _get_spatial_dims(self._obj, var=var)
                 resampled_dataset[var] = (
                     self._obj[var]
                     .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
@@ -196,6 +198,7 @@ class RasterDataset(XRasterBase):
         miny: float,
         maxx: float,
         maxy: float,
+        *,
         constant_values: Union[
             float, tuple[int, int], Mapping[Any, tuple[int, int]], None
         ] = None,
@@ -229,7 +232,7 @@ class RasterDataset(XRasterBase):
         padded_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
             try:
-                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                x_dim, y_dim = _get_spatial_dims(self._obj, var=var)
                 padded_dataset[var] = (
                     self._obj[var]
                     .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
@@ -253,6 +256,7 @@ class RasterDataset(XRasterBase):
         miny: float,
         maxx: float,
         maxy: float,
+        *,
         auto_expand: Union[bool, int] = False,
         auto_expand_limit: int = 3,
         crs: Optional[Any] = None,
@@ -294,7 +298,7 @@ class RasterDataset(XRasterBase):
         clipped_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
             try:
-                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                x_dim, y_dim = _get_spatial_dims(self._obj, var=var)
                 clipped_dataset[var] = (
                     self._obj[var]
                     .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
@@ -323,6 +327,7 @@ class RasterDataset(XRasterBase):
         self,
         geometries: Iterable,
         crs: Optional[Any] = None,
+        *,
         all_touched: bool = False,
         drop: bool = True,
         invert: bool = False,
@@ -383,7 +388,7 @@ class RasterDataset(XRasterBase):
         clipped_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
             try:
-                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                x_dim, y_dim = _get_spatial_dims(self._obj, var=var)
                 clipped_dataset[var] = (
                     self._obj[var]
                     .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
@@ -429,7 +434,7 @@ class RasterDataset(XRasterBase):
         interpolated_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
             try:
-                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                x_dim, y_dim = _get_spatial_dims(self._obj, var=var)
                 interpolated_dataset[var] = (
                     self._obj[var]
                     .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
@@ -448,6 +453,7 @@ class RasterDataset(XRasterBase):
     def to_raster(
         self,
         raster_path: Union[str, os.PathLike],
+        *,
         driver: Optional[str] = None,
         dtype: Optional[Union[str, numpy.dtype]] = None,
         tags: Optional[dict[str, str]] = None,
@@ -503,35 +509,58 @@ class RasterDataset(XRasterBase):
             is True. Otherwise None is returned.
 
         """
+        # pylint: disable=too-many-locals
         variable_dim = f"band_{uuid4()}"
         data_array = self._obj.to_array(dim=variable_dim)
         # ensure raster metadata preserved
-        scales = []
-        offsets = []
-        nodatavals = []
+        attr_scales = []
+        attr_offsets = []
+        attr_nodatavals = []
+        encoded_scales = []
+        encoded_offsets = []
+        encoded_nodatavals = []
         band_tags = []
         long_name = []
         for data_var in data_array[variable_dim].values:
-            scales.append(self._obj[data_var].attrs.get("scale_factor", 1.0))
-            offsets.append(self._obj[data_var].attrs.get("add_offset", 0.0))
+            try:
+                encoded_scales.append(self._obj[data_var].encoding["scale_factor"])
+            except KeyError:
+                attr_scales.append(self._obj[data_var].attrs.get("scale_factor", 1.0))
+            try:
+                encoded_offsets.append(self._obj[data_var].encoding["add_offset"])
+            except KeyError:
+                attr_offsets.append(self._obj[data_var].attrs.get("add_offset", 0.0))
             long_name.append(self._obj[data_var].attrs.get("long_name", data_var))
-            nodatavals.append(self._obj[data_var].rio.nodata)
+            if self._obj[data_var].rio.encoded_nodata is not None:
+                encoded_nodatavals.append(self._obj[data_var].rio.encoded_nodata)
+            else:
+                attr_nodatavals.append(self._obj[data_var].rio.nodata)
             band_tags.append(self._obj[data_var].attrs.copy())
-        data_array.attrs["scales"] = scales
-        data_array.attrs["offsets"] = offsets
+        if encoded_scales:
+            data_array.encoding["scales"] = encoded_scales
+        else:
+            data_array.attrs["scales"] = attr_scales
+        if encoded_offsets:
+            data_array.encoding["offsets"] = encoded_offsets
+        else:
+            data_array.attrs["offsets"] = attr_offsets
         data_array.attrs["band_tags"] = band_tags
         data_array.attrs["long_name"] = long_name
 
+        use_encoded_nodatavals = bool(encoded_nodatavals)
+        nodatavals = encoded_nodatavals if use_encoded_nodatavals else attr_nodatavals
         nodata = nodatavals[0]
         if (
             all(nodataval == nodata for nodataval in nodatavals)
             or numpy.isnan(nodatavals).all()
         ):
-            data_array.rio.write_nodata(nodata, inplace=True)
+            data_array.rio.write_nodata(
+                nodata, inplace=True, encoded=use_encoded_nodatavals
+            )
         else:
             raise RioXarrayError(
                 "All nodata values must be the same when exporting to raster. "
-                f"Current values: {nodatavals}"
+                f"Current values: {attr_nodatavals}"
             )
         if self.crs is not None:
             data_array.rio.write_crs(self.crs, inplace=True)
