@@ -1,4 +1,5 @@
 import os
+from functools import partial
 
 import pytest
 import rasterio
@@ -14,6 +15,8 @@ xarray.set_options(warn_for_unclosed_files=True)
 TEST_DATA_DIR = os.path.join(os.path.dirname(__file__), "test_data")
 TEST_INPUT_DATA_DIR = os.path.join(TEST_DATA_DIR, "input")
 TEST_COMPARE_DATA_DIR = os.path.join(TEST_DATA_DIR, "compare")
+RASTERIO_GE_14 = version.parse(rasterio.__version__) >= version.parse("1.4.0")
+RASTERIO_GE_143 = version.parse(rasterio.__version__) >= version.parse("1.4.3")
 GDAL_GE_36 = version.parse(rasterio.__gdal_version__) >= version.parse("3.6.0")
 GDAL_GE_361 = version.parse(rasterio.__gdal_version__) >= version.parse("3.6.1")
 GDAL_GE_364 = version.parse(rasterio.__gdal_version__) >= version.parse("3.6.4")
@@ -102,9 +105,7 @@ def _assert_xarrays_equal(
             assert unwanted_attr not in input_xarray.attrs
 
 
-def open_rasterio_engine(file_name_or_object, **kwargs):
-    xarray = pytest.importorskip("xarray", minversion="0.18")
-    return xarray.open_dataset(file_name_or_object, engine="rasterio", **kwargs)
+open_rasterio_engine = partial(xarray.open_dataset, engine="rasterio")
 
 
 @pytest.fixture(
